package dev.fitko.fitconnect.client.cmd;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.client.factory.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.factory.ClientFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.stream.Collectors;

public final class CommandLineRunner {

    private static final String LOGO = "/splash_screen_banner.txt";
    public static final String DEFAULT_CONFIG_NAME = "config.yml";

    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineRunner.class);

    private CommandLineRunner() {
    }

    public static void main(final String[] args) {
        printSplashScreen();
        run(args);
    }

    private static void run(final String[] args) {
        final ApplicationConfig config = loadConfig();
        getCommandLineClient(config).run(args);
    }

    private static ApplicationConfig loadConfig() {
        try {
            return ApplicationConfigLoader.loadConfig(DEFAULT_CONFIG_NAME);
        } catch (final Exception e) {
            LOGGER.info("Could not find default config {}", DEFAULT_CONFIG_NAME);
            return null;
        }
    }

    private static CommandLineClient getCommandLineClient(final ApplicationConfig config) {
        final var senderClient = config == null ? ClientFactory.senderClient() : ClientFactory.senderClient(config);
        final var subscriberClient = config == null ? ClientFactory.subscriberClient() : ClientFactory.subscriberClient(config);
        return new CommandLineClient(senderClient, subscriberClient);
    }

    private static void printSplashScreen() {
        LOGGER.info("{}", getSplashScreenResource());
    }

    private static String getSplashScreenResource() {
        final InputStream is = CommandLineRunner.class.getResourceAsStream(LOGO);
        return new BufferedReader(new InputStreamReader(is)).lines().collect(Collectors.joining("\n"));
    }
}
